class Pettanr.Views.Comic.PlayModule
class Pettanr.Views.Comic.PlayModule.Header extends Pettanr.Views.Show.Header
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Comic.PlayModule.Owner extends Pettanr.Views.Show.Owner
  
  initialize: (options) ->
    super(options)
  
class Pettanr.Views.Comic.PlayModule.Footer extends Backbone.View
  tagName: 'table'
  className: 'no-border'
  
  initialize: (options) ->
    @panel = options.panel
    @scroll_panel = options.scroll_panel
    @operators = options.operators
  
  render: () ->
    this.$el.html('')
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@panel.get('updated_at')))
    #
    this.$el.append(
      (new Pettanr.Views.Panel.Icon({item: @scroll_panel, half: true})).render().el
    )
    this.$el.append(
      (new Pettanr.Views.Author.Icon({item: @panel.author(), half: true})).render().el
    )
    this.$el.append(Pettanr.AppHelper.distance_of_time_in_words_to_now(@scroll_panel.get('updated_at')))
    #
    if @panel.is_own(@operators)
      this.$el.append('')
    this
  
class Pettanr.Views.Comic.PlayModule.Stories extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @items = options.items
    @operators = options.operators
  
  render: () ->
    this.$el.html('')
    _.each @items, (comic_story) =>
      return if not comic_story.has_story()
      story = comic_story.story()
      story.fetch({cache: true}).done =>
        this.$el.append('title')
        this.$el.append('order')
    this
  
class Pettanr.Views.Comic.PlayModule.Body extends Backbone.View
  tagName: 'div'
  
  initialize: (options) ->
    @parent = options.parent
    @list = options.list
    @operators = options.operators
    @items = @list.items()
    if @items.length < 1
      @render_empty()
    else
      @render()
  
  render_empty: () ->
    this.$el.html('')
    this.$el.append((new Tag.H2({content: I18n.t('comics.play.empty')})).render().el)
    this
  
  render: () ->
    this.$el.html('')
    panels = new Pettanr.Views.Comic.PlayModule.Stories({
      parent: this,
      items: @items,
      operators: @operators
    })
    this.$el.append(panels.render().el)
    this
  
class Pettanr.Views.Comic.Play extends Backbone.View
  tagName: 'div'
  className: 'show'
  
  initialize: (options) ->
    @item = options.item
    @list = options.list
    @operators = options.operators
    icon_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id')})
    caption_url = Pettanr.url(@item.table_name(), 'play', {id: @item.get('id')})
    prof_url = Pettanr.url(@item.table_name(), 'show', {id: @item.get('id'), format: 'prof'})
    @header = new Pettanr.Views.Comic.PlayModule.Header({
      item: @item, 
      operators: @operators,
      caption: @item.get('title'), 
      icon_url: icon_url,
      caption_url: caption_url,
      prof_url: prof_url
    })
    @author = new Pettanr.Views.Show.HeaderAuthor({item: @item})
    @body = new Pettanr.Views.Comic.PlayModule.Body({
      parent: this,
      list: @list, 
      operators: @operators
    })
    @owner = new Pettanr.Views.Comic.PlayModule.Owner({item: @item, operators: @operators})
  
  render: () ->
    this.$el.html('')
    this.$el.append(@header.render().el)
    this.$el.append(@author.render().el)
    this.$el.append(@body.render().el)
    this.$el.append(@owner.render().el) if @item.is_own(@operators)
    this
  
