class SheetPanel < Peta::Element
  load_manifest
  belongs_to :panel
  belongs_to :sheet
  accepts_nested_attributes_for :panel, :allow_destroy => true
  #alias_attribute :attr_y, :y
  
  validates :sheet_id, :numericality => {:allow_blank => true}
  validates :panel_id, :numericality => {:allow_blank => true}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :z, :presence => true, :numericality => {:greater_than => 0}
  validates :t, :presence => true, :numericality => {:greater_than_or_equal_to => 0}
  
  def y
    self.attributes['y']
  end
  
  def supply_default
    self.x = 15
    self.y = 15
    if self.sheet
      self.t = self.sheet.new_t 
      self.z = self.sheet.new_z 
    else
      self.sheet_id = nil
      self.panel_id = nil
      self.z = 1
      self.t = nil
    end
  end
  
  def self.by_author_list_includes
    {
      :sheet => {
        :author => {}
      }
    }
  end
  
  def self.show_opt
    {:include => {
      :sheet => {
        :author => {}
      }, 
      :panel => {
        :author => {}, 
        :panel_pictures => {:picture => {:artist => {}, :license => {}}}, 
        :speech_balloons =>{:balloon => {}, :speech => {}}
      }
    }}
  end
  
  def elements
    self.panel.elements
  end
  
  def sheet_panel_as_json au
    panel_include = if self.panel and self.panel.visible?(au)
      {:include => {}, :methods => :elements}
    else
      {:include => {}}
    end
    self.to_json({:include => {:sheet => {:include => {:author => {}}}, :panel => panel_include}})
  end
  
  def self.list_as_json_text ary, au
    '[' + ary.map {|i| i.sheet_panel_as_json(au) }.join(',') + ']'
  end
  
  def licensed_pictures
    if self.panel
      self.panel.licensed_pictures
    else
      {}
    end
  end
  
    def self.panelize elements_attributes
      elements_attributes = [elements_attributes] unless elements_attributes.is_a?(Array)
      hash = {}
      index = 0
      elements_attributes.each do |element_attributes|
        hash[self.to_s.tableize + '_attributes'] ||= {}
        n = if element_attributes['id']
          element_attributes['id'].to_s
        else
          index += 1
          'new' + index.to_s 
        end
        hash[self.to_s.tableize + '_attributes'][n] = element_attributes
      end
      hash
    end
    
    def copy_attributes
      r = self.attributes
      r.delete 'id'
      r.delete 'sheet_id'
      r.delete 'panel_id'   # create panel
      r.delete 'created_at'
      r.delete 'updated_at'
      r
    end
  
  def copyable?
    if self.panel and self.panel.publish? == false
      false
    else
      true
    end
  end
  
  def panel_attributes
    if self.panel
      {'panel_attributes' => self.panel.copy}
    else
      {}
    end
  end
  
  def allow? operators
    return nil if self.sheet_id == nil or self.panel_id == nil
    self.sheet.own?(operators) and self.panel.usable?(operators)
  end
  
  def store operators, old_t = nil
    res = false
    self.class.transaction do
      case self.allow? operators
      when true
        self.rotate old_t
      when false
        raise ActiveRecord::Forbidden
      else
      end
      res = self.save
      raise ActiveRecord::Rollback unless res
      res = self.class.validate_t(self.sheet_id) 
      unless res
        self.errors.add :t, 'unserialized'
        raise ActiveRecord::Rollback 
      end
    end
    res
  end
  
end
