class Speech < Peta::Element
  load_manifest
  belongs_to :speech_balloon
  belongs_to :speech_balloon_template
  belongs_to :writing_format
  
  validates :speech_balloon_id, :numericality => {:allow_blank => true}
  validates :speech_balloon_template_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :speech_balloon_template_module_name, :presence => true, :length => {:maximum => 50}
  validates :writing_format_id, :presence => true, :numericality => true, :existence => {:both => false}
  validates :writing_format_module_name, :presence => true, :length => {:maximum => 50}
  validates :font_size, :presence => true, :numericality => {:only_integer => false}
  validates :text_align, :presence => true, :numericality => true, :inclusion => { :in => 0..3 }
  validates :fore_color, :presence => true, :numericality => {:greater_than_or_equal_to => 0, :less_than => 0x1000000}
  validates :x, :presence => true, :numericality => true
  validates :y, :presence => true, :numericality => true
  validates :width, :presence => true, :numericality => true, :natural_number => true
  validates :height, :presence => true, :numericality => true, :natural_number => true
  validates :quotes, :length => {:maximum => 15}, :quotes_even => true
  validates :speech_balloon_template_settings, :boost => {:boost_name => :speech_balloon_template}
  #validates :writing_format_settings
  
  @@text_align_texts = ['left', 'left', 'right', 'center']
  
  def y
    self.attributes['y']
  end
  
  def supply_default
    self.x = 0
    self.y = 0
    self.width = 100
    self.height = 100
    self.fore_color = 0
  end
  
  def symbol_option
    self.speech_balloon.speech_balloon_template.symbol_option
  end
  
  def outer_style
    {
      'top' => self.y, 'left' => self.x, 
      'width' => self.width, 'height' => self.height
    }
  end
  
  def inner_style
    {
      'font-size' => self.font_size.to_s + 'em',
      'text-align' => self.text_align_text, 
      'color' => '#' + format("%06x", self.fore_color)
    }
  end
  
  def text_align_text
    @@text_align_texts[self.text_align]
  end
  
  def self.public_list_where list
    'panels.publish > 0'
  end
  
  def self.by_author_list_includes
    {
      :speech_balloon => {
        :panel => {
          :author => {}
        }
      }
    }
  end
  
  def self.show_opt
    {:include => {:speech_balloon => {:panel => {:author => {}}, :balloon => {}, :speech_balloon_template => {} }}}
  end
  
  def self.panelize speech_attributes
    {'speech_attributes' => speech_attributes}
  end
  
  def scenario
    self.boosts 'read'
    self.render
  end
  
  def plain_scenario
    self.content + "\n"
  end
  
  def feed
    ERB::Util.html_escape(self.content)
  end
  
end
