module Locmare
  module ListGroupModule
    module LibModule
      module Pager
        class Default
          
          attr :page_status, :options, :window_size, 
            :total, :current_page, :per_page, :total_page, 
            :template_dir, :template_name
          
          def initialize page_status, options
            @page_status = page_status
            @options = options
            @total = @page_status.total
            
            @window_size = 3
            @template_dir = 'templates/r/pager/'
            @template_name = @template_dir + 'default'
            @controller = Manifest.manifest.controllers[@options['controller'].to_s]
            @action = @controller.actions[@options['action'].to_s]
            @action = @action.original if @action.original
            
            @current_page = @page_status.page
            @per_page = @page_status.limit
            @total_page = @page_status.total_page
          end
          
          def range
            f = if self.has_previous_page_gap
              @current_page - @window_size
            else
              1
            end
            t = if self.has_next_page_gap
              @current_page + @window_size
            else
              @total_page
            end
            f..t
          end
          
          def has_previous_page
            @current_page > 1
          end
          
          def has_next_page
            @current_page < @total_page
          end
          
          def has_previous_page_gap
            @current_page > @window_size + 1
          end
          
          def has_next_page_gap
            @total_page - @current_page > @window_size
          end
          
          def url page
            params = {}
            @options.each do |k, v|  # deep copy
              params[k] = v
            end
            params['page'] = page
            params['page_size'] = @per_page
            @action.url params
          end
          
        end
        
        class Offset
          
          attr :page_status, :options, :window_size, 
            :total, :current_page, :per_page, :total_page, 
            :template_dir, :template_name
          
          def initialize page_status, options
            @page_status = page_status
            @options = options
            @total = @page_status.total
            
            @window_size = 3
            @template_dir = 'templates/r/pager/'
            @template_name = @template_dir + 'offset'
            @controller = Manifest.manifest.controllers[@options['controller'].to_s]
            @action = @controller.actions[@options['action'].to_s]
            
            @current_page = @page_status.page
            @per_page = @page_status.limit
            @total_page = @page_status.total_page
          end
          
          def ranges
            ranges = []
            (-@window_size..@window_size).each do |d|
              next unless r = self.diff_offset(d)
              ranges << r
            end
            ranges
          end
          
          def has_previous_page
            @current_page > 1
          end
          
          def has_next_page
            @current_page < @total_page
          end
          
          def has_previous_page_gap
            @current_page > @window_size + 1
          end
          
          def has_next_page_gap
            @total_page - @current_page > @window_size
          end
          
          def diff_offset d
            f = @page_status.offset + d * @per_page
            t = f + @per_page - 1
            return nil if t < 0 or f >= @total
            f = 0 if f < 0
            t = @total - 1 if t >= @total
            {
              'offset' => f, 'count' => t - f + 1, 
              'page' => @current_page + d, 'distance' => d
            }
          end
          
          def url range
            params = {}
            @options.each do |k, v|  # deep copy
              params[k] = v
            end
            params['offset'] = range['offset']
            params['count'] = range['count']
            @action.url params
          end
          
        end
      end
    end
  end
end
